declare type StateType = 'Task' | 'Map' | 'Pass' | 'Wait' | 'Choice' | 'Succeed' | 'Fail';

interface BaseState {
    Type: StateType;
    Comment?: string;
}

declare type JSONPrimitiveValue = null | boolean | number | string;
declare type JSONArray = (JSONPrimitiveValue | JSONArray | JSONObject)[];
declare type JSONObject = {
    [key: string]: JSONPrimitiveValue | JSONObject | JSONArray;
};
declare type JSONValue = JSONPrimitiveValue | JSONObject | JSONArray;

declare type PayloadTemplate = JSONObject;
interface CanHaveInputPath {
    InputPath?: string | null;
}
interface CanHaveParameters {
    Parameters?: PayloadTemplate;
}
interface CanHaveResultSelector {
    ResultSelector?: PayloadTemplate;
}
interface CanHaveResultPath {
    ResultPath?: string | null;
}
interface CanHaveOutputPath {
    OutputPath?: string | null;
}

declare type StringOperatorNames = 'StringEquals' | 'StringLessThan' | 'StringGreaterThan' | 'StringLessThanEquals' | 'StringGreaterThanEquals' | 'StringMatches';
declare type StringPathOperatorNames = 'StringEqualsPath' | 'StringLessThanPath' | 'StringGreaterThanPath' | 'StringLessThanEqualsPath' | 'StringGreaterThanEqualsPath';
declare type NumericOperatorNames = 'NumericEquals' | 'NumericLessThan' | 'NumericGreaterThan' | 'NumericLessThanEquals' | 'NumericGreaterThanEquals';
declare type NumericPathOperatorNames = 'NumericEqualsPath' | 'NumericLessThanPath' | 'NumericGreaterThanPath' | 'NumericLessThanEqualsPath' | 'NumericGreaterThanEqualsPath';
declare type BooleanOperatorNames = 'BooleanEquals';
declare type BooleanPathOperatorNames = 'BooleanEqualsPath';
declare type TimestampOperatorNames = 'TimestampEquals' | 'TimestampLessThan' | 'TimestampGreaterThan' | 'TimestampLessThanEquals' | 'TimestampGreaterThanEquals';
declare type TimestampPathOperatorNames = 'TimestampEqualsPath' | 'TimestampLessThanPath' | 'TimestampGreaterThanPath' | 'TimestampLessThanEqualsPath' | 'TimestampGreaterThanEqualsPath';
declare type TypeTestOperatorNames = 'IsNull' | 'IsPresent' | 'IsNumeric' | 'IsString' | 'IsBoolean' | 'IsTimestamp';
declare type StringComparisonOperator = {
    [P in StringOperatorNames]?: string;
};
declare type StringPathComparisonOperator = {
    [P in StringPathOperatorNames]?: string;
};
declare type NumericComparisonOperator = {
    [P in NumericOperatorNames]?: number;
};
declare type NumericPathComparisonOperator = {
    [P in NumericPathOperatorNames]?: string;
};
declare type BooleanComparisonOperator = {
    [P in BooleanOperatorNames]?: boolean;
};
declare type BooleanPathComparisonOperator = {
    [P in BooleanPathOperatorNames]?: string;
};
declare type TimestampComparisonOperator = {
    [P in TimestampOperatorNames]?: string;
};
declare type TimestampPathComparisonOperator = {
    [P in TimestampPathOperatorNames]?: string;
};
declare type TypeTestComparisonOperator = {
    [P in TypeTestOperatorNames]?: true;
};
declare type ComparisonOperator = StringComparisonOperator | StringPathComparisonOperator | NumericComparisonOperator | NumericPathComparisonOperator | BooleanComparisonOperator | BooleanPathComparisonOperator | TimestampComparisonOperator | TimestampPathComparisonOperator | TypeTestComparisonOperator;
declare type BaseDataTestExpression = {
    Variable: string;
};
declare type DataTestExpression = BaseDataTestExpression & ComparisonOperator;
declare type BooleanExpression = {
    And?: ChoiceRuleWithoutNext[];
    Or?: ChoiceRuleWithoutNext[];
    Not?: ChoiceRuleWithoutNext;
};
declare type ChoiceRuleWithoutNext = DataTestExpression | BooleanExpression;
declare type ChoiceRule = (DataTestExpression | BooleanExpression) & {
    Next: string;
};
interface BaseChoiceState extends BaseState, CanHaveInputPath, CanHaveOutputPath {
    Type: 'Choice';
    Choices: ChoiceRule[];
    Default?: string;
}
declare type ChoiceState = BaseChoiceState;

interface EndableState extends BaseState {
    Type: Exclude<StateType, 'Choice' | 'Succeed' | 'Fail'>;
    End: true;
}
interface SucceedOrFailState extends BaseState {
    Type: Extract<StateType, 'Succeed' | 'Fail'>;
}
declare type TerminalState = EndableState | SucceedOrFailState;

interface BaseFailState extends BaseState {
    Type: 'Fail';
    Cause?: string;
    Error?: string;
}
declare type FailState = TerminalState & BaseFailState;

declare type Retrier = {
    ErrorEquals: string[];
    IntervalSeconds?: number;
    MaxAttempts?: number;
    BackoffRate?: number;
};
interface RetryableState {
    Retry?: Retrier[];
}
declare type Catcher = {
    ErrorEquals: string[];
    Next: string;
    ResultPath?: string;
};
interface CatchableState {
    Catch?: Catcher[];
}

interface NextableState extends BaseState {
    Type: Exclude<StateType, 'Choice' | 'Succeed' | 'Fail'>;
    Next: string;
}
declare type IntermediateState = NextableState;

interface BaseMapState extends BaseState, CanHaveInputPath, CanHaveParameters, CanHaveResultSelector, CanHaveResultPath, CanHaveOutputPath, RetryableState, CatchableState {
    Type: 'Map';
    Iterator: Omit<StateMachineDefinition, 'Version' | 'TimeoutSeconds'>;
    ItemsPath?: string;
    MaxConcurrency?: number;
}
declare type MapState = (IntermediateState | TerminalState) & BaseMapState;

interface BasePassState extends BaseState, CanHaveInputPath, CanHaveParameters, CanHaveResultPath, CanHaveOutputPath {
    Type: 'Pass';
    Result?: JSONValue;
}
declare type PassState = (IntermediateState | TerminalState) & BasePassState;

interface BaseSucceedState extends BaseState, CanHaveInputPath, CanHaveOutputPath {
    Type: 'Succeed';
}
declare type SucceedState = TerminalState & BaseSucceedState;

interface BaseTaskState extends BaseState, CanHaveInputPath, CanHaveParameters, CanHaveResultSelector, CanHaveResultPath, CanHaveOutputPath, RetryableState, CatchableState {
    Type: 'Task';
    Resource: string;
}
declare type TaskState = (IntermediateState | TerminalState) & BaseTaskState;

interface BaseWaitState extends BaseState, CanHaveInputPath, CanHaveOutputPath {
    Type: 'Wait';
    Seconds?: number;
    Timestamp?: string;
    SecondsPath?: string;
    TimestampPath?: string;
}
declare type WaitState = (IntermediateState | TerminalState) & BaseWaitState;

declare type AllStates = TaskState | MapState | PassState | WaitState | ChoiceState | SucceedState | FailState;

interface StateMachineDefinition {
    States: Record<string, AllStates>;
    StartAt: string;
    Comment?: string;
    Version?: string;
    TimeoutSeconds?: number;
}

declare type TaskStateResourceLocalHandler = {
    [taskStateName: string]: (input: JSONValue) => Promise<JSONValue>;
};
declare type WaitStateTimeOverride = {
    [waitStateName: string]: number;
};
interface Overrides {
    taskResourceLocalHandlers?: TaskStateResourceLocalHandler;
    waitTimeOverrides?: WaitStateTimeOverride;
}
interface ValidationOptions {
    readonly checkPaths?: boolean;
    readonly checkArn?: boolean;
}
interface RunOptions {
    overrides?: Overrides;
    noThrowOnAbort?: boolean;
}

declare class StateMachine {
    /**
     * The structure of the State Machine as represented by the Amazon States Language.
     */
    private readonly definition;
    /**
     * Options to control whether to apply certain validations to the state machine definition.
     */
    private readonly validationOptions;
    /**
     * Constructs a new state machine.
     * @param definition The state machine definition defined using the Amazon States Language (https://states-language.net/spec.html).
     * @param validationOptions Options to control whether to apply certain validations to the definition.
     * These options also apply to state machines defined in  the `Iterator` field of `Map` states.
     */
    constructor(definition: StateMachineDefinition, validationOptions?: ValidationOptions);
    /**
     * Executes the state machine, running through the states specified in the definition.
     * If the execution fails, the result will throw an `ExecutionError` explaining why the
     * execution failed.
     *
     * By default, if the execution is aborted, the result will throw an `ExecutionAbortedError`. This behavior can be changed by setting
     * the `noThrowOnAbort` option to `true`, in which case the result will be `null`.
     *
     * @param input The input to pass to this state machine execution.
     * @param options Miscellaneous options to control certain behaviors of the execution.
     */
    run(input: JSONValue, options?: RunOptions): {
        abort: () => void;
        result: Promise<JSONValue>;
    };
    /**
     * Helper method that handles the execution of the machine states and the transitions between them.
     */
    private execute;
}

/**
 * Represents the failure of a state machine execution.
 */
declare class ExecutionError extends Error {
    #private;
    constructor(caughtError: Error);
    get getWrappedError(): Error;
}

/**
 * Represents the abortion of a state machine execution.
 */
declare class ExecutionAbortedError extends Error {
    constructor();
}

export { ExecutionAbortedError, ExecutionError, StateMachine };
